#!/usr/bin/python

""" 
This script will resolve all units in error state,
remove units and remove all services.
You can run ' ./juju-reset.py machines' to have all machines
removed as well
"""

import yaml
from subprocess import check_output, CalledProcessError
import sys
from shlex import split

def py_status():
    status = check_output(split('juju status -e %s' % environment))
    py_status = yaml.load(status)
    return py_status

def get_services(status):
    if 'services' in status:
        return status['services'].keys()
    else:
        return []

def get_units(status, service):
    # return dictionary of units
    if 'units' in status['services'][service]:
        return status['services'][service]['units']
    else:
        return {}

def get_machine_from_unit(unit):
    if 'machine' in unit:
        return unit['machine']
    else:
        return []

def juju(command, name):
    try:
        print check_output(split('juju %s %s -e %s' % \
                           (command, name, environment))) + '.'
    except CalledProcessError:
        raise

def unit_resolved(unit, max_retries=3):
    # unit is passed as a tuple (key,value)
    i = 0
    while (unit[1]['agent-state'] != 'started'):
        try:
            juju('resolved', unit[0])
        except CalledProcessError:
            print("No need to resolve unit %s %s" % \
                    (unit[0], unit[1]['agent-state']))
        i += 1
        if i >= max_retries:
            break

def destroy_units(units_list):
    for unit in units_list:
        try:
            juju('destroy-unit', unit)
        except CalledProcessError:
            print "No unit to be destroyed: %s" % unit

def destroy_services(services):
    for service in services:
        try:
            juju('destroy-service', service)
        except CalledProcessError:
            print "No service to be destroyed: %s" % service

def destroy_machines(machines):
    if len(machines) > 0:
        juju('destroy-machine', ' '.join(machines))


def juju_destroy(stat, services_off=True, machines_off=False):
    # main method
    services = get_services(stat)
    units = {}
    machines = []
    for serv in services:
        units.update(get_units(stat, serv))
    print "Services %s and units %s" % (services, units.keys())
    for u in units.items():
        print "Unit to resolve %s" % u[0]
        machines.append(get_machine_from_unit(u))
        unit_resolved(u)
    
    if services_off:
        destroy_units(units.keys())
        destroy_services(services)

    if machines_off:
        destroy_machines(machines)


args = sys.argv
environment = 'local'
if len(args) > 2:
    environment = args[1]
if 'machines' in args:
    machines_off = True
else:
    machines_off = False
stat = py_status()
juju_destroy(stat, machines_off=machines_off)
print py_status()
